package me.hammerle.supersnuvi.tiles;

import java.util.HashSet;
import me.hammerle.supersnuvi.entity.Entity;
import me.hammerle.supersnuvi.entity.EntityBuilder;
import me.hammerle.supersnuvi.gamelogic.Level;
import me.hammerle.supersnuvi.util.CollisionBox;
import me.hammerle.supersnuvi.util.CollisionObject;
import me.hammerle.supersnuvi.util.Face;
import me.hammerle.supersnuvi.util.SoundUtils;
import me.hammerle.supersnuvi.util.Utils;

public class CrumblingStoneTile extends BaseTile
{
    private final HashSet<Long> states;
    
    public CrumblingStoneTile() 
    {
        super(0.1875f, 0.125f, 0.25f, 0.1875f);
        states = new HashSet<>();
        super.setCollisionBox(CollisionBox.createScaledTileBox(0.0f, 0.0f, 1.0f, 0.6f));
        super.setMovementBox(CollisionBox.createScaledTileBox(0.0f, 0.0f, 1.0f, 0.6f));
    }

    @Override
    public CollisionObject getCollisionBox(int x, int y) 
    {
        if(states.contains(getKey(x, y)))
        {
            return CollisionObject.NULL_BOX;
        }
        return super.getCollisionBox(x, y);
    }

    @Override
    public CollisionObject getMovementBox(int x, int y) 
    {
        if(states.contains(getKey(x, y)))
        {
            return CollisionObject.NULL_BOX;
        }
        return super.getMovementBox(x, y);
    }

    @Override
    public void onEntityCollide(Entity ent, int x, int y, Face face) 
    {
        super.onEntityCollide(ent, x, y, face);
        if(face == Face.UP)
        {
            if(states.add(getKey(x, y)))
            {
                ent.getLevel().updateTile(x, y);
                Level l = ent.getLevel();               
                l.spawnEntity(EntityBuilder.buildCrumblingStone(l, Utils.toCoord(x), Utils.toCoord(y)));
                SoundUtils.playSound(SoundUtils.Sound.STONE_CRUMBLING);
            }
        }
    }

    @Override
    public boolean shouldRender(int x, int y)
    {
        return !states.contains(getKey(x, y));
    }

    @Override
    public void reset() 
    {
        states.clear();
    } 
}
