package me.hammerle.supersnuvi.util;

import me.hammerle.supersnuvi.tiles.Tile;

public class CollisionBox extends CollisionObject
{
    private float minX;
    private float maxX;
    private float minY;
    private float maxY;
    
    private float cMinX;
    private float cMaxX;
    private float cMinY;
    private float cMaxY;
    
    public CollisionBox(float x1, float y1, float x2, float y2)
    {
        minX = Math.min(x1, x2);
        maxX = Math.max(x1, x2);
        minY = Math.min(y1, y2);
        maxY = Math.max(y1, y2);
        cMinX = minX;
        cMaxX = maxX;
        cMinY = minY;
        cMaxY = maxY;
    }
    
    public static CollisionObject createScaledTileBox(float x1, float y1, float x2, float y2)
    {
        return new CollisionBox(x1 * Tile.SIZE, y1 * Tile.SIZE, x2 * Tile.SIZE, y2 * Tile.SIZE);
    }
    
    @Override
    public CollisionObject copy()
    {
        return new CollisionBox(cMinX, cMinY, cMaxX, cMaxY);
    }
    
    @Override
    public void save()
    {
        minX = cMinX;
        maxX = cMaxX;
        minY = cMinY;
        maxY = cMaxY;
    }
    
    @Override
    public CollisionObject reset()
    {
        cMinX = minX;
        cMaxX = maxX;
        cMinY = minY;
        cMaxY = maxY;
        return this;
    }

    @Override
    public float getWidth() 
    {
        return maxX - minX;
    }
    
    @Override
    public float getHeight() 
    {
        return maxY - minY;
    }
    
    @Override
    public float getMinX() 
    {
        return cMinX;
    }

    @Override
    public float getMaxX() 
    {
        return cMaxX;
    }

    @Override
    public float getMinY() 
    {
        return cMinY;
    }

    @Override
    public float getMaxY() 
    {
        return cMaxY;
    }
   
    @Override
    public CollisionObject expand(float x, float y)
    {
        if(x < 0.0f)
        {
            cMinX += x;
        }
        else
        {
            cMaxX += x;
        }
        if(y < 0.0f)
        {
            cMinY += y;
        }
        else
        {
            cMaxY += y;
        }
        return this;
    }
    
    @Override
    public CollisionObject offsetX(float x)
    {
        cMinX += x;
        cMaxX += x;
        return this;
    }
    
    @Override
    public CollisionObject offsetY(float y)
    {
        cMinY += y;
        cMaxY += y;
        return this;
    }
    
    @Override
    public boolean isColliding(CollisionObject cb)
    {
        switch(cb.getType())
        {
            case BOX:
                return cMaxX > cb.getMinX() && cb.getMaxX() > cMinX && cMaxY > cb.getMinY() && cb.getMaxY() > cMinY;
            case LINE:
                float x1 = cb.getMinX();
                float y1 = cb.getMinY();
                float x2 = cb.getMaxX();
                float y2 = cb.getMaxY();
                return CollisionLine.intersect(x1, y1, x2, y2, cMinX, cMinY, cMaxX, cMinY) || 
                        CollisionLine.intersect(x1, y1, x2, y2, cMaxX, cMinY, cMaxX, cMaxY) ||
                        CollisionLine.intersect(x1, y1, x2, y2, cMaxX, cMaxY, cMinX, cMaxY) ||
                        CollisionLine.intersect(x1, y1, x2, y2, cMinX, cMaxY, cMinX, cMinY);
        }
        return false;
        
    }

    @Override
    public String toString() 
    {
        StringBuilder sb = new StringBuilder("box[x1 = ");
        sb.append(cMinX);
        sb.append(", y1 = ");
        sb.append(cMinY);
        sb.append(", x2 = ");
        sb.append(cMaxX);
        sb.append(", y2 = ");
        sb.append(cMaxY);
        sb.append(']');
        return sb.toString();
    } 

    @Override
    public Type getType()
    {
        return Type.BOX;
    }
}
