package me.hammerle.supersnuvi.util;

import java.io.File;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.DataLine;
import me.hammerle.supersnuvi.Game;

public class SoundUtils 
{
    private static AudioFormat WAV;
    private static DataLine.Info INFO;
    
    static
    {
        try
        {
            WAV = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, 44100f, 24, 1, 3, 44100.0f, false);
            INFO = new DataLine.Info(Clip.class, WAV);
        }
        catch(Exception ex)
        {
            WAV = null;
            INFO = null;
            ex.printStackTrace();
        }
    }
    
    public enum Sound
    {
        DASH("dash", true),
        DODGE("dodge", true), 
        LONDONER_JUMP("londoner_jump", true), 
        MIRROR_BREAK("mirror_break", true), 
        MIRROR_CRACK("mirror_crack", true), 
        SWORD_PULL("sword_pull", true),
        SWORD_SHEATH("sword_sheath", true), 
        SWORD_SLASH("sword_slash", true), 
        
        COLLECT("collect", true), 
        JUMP("jump", true), 
        JUMP_ON_BOUNCE_SHROOM("jump_on_bounce_shroom", false), 
        STONE_CRUMBLING("stone_crumbling", true), 
        WALK("walk", false), 
        WALK_WATER("walk_water", false),
        MENU_MUSIC("menu_music", false), 
        SONG_1("song1", false);
        
        private final boolean reset;
        private Clip clip;
        
        Sound(String path, boolean reset)
        {
            this.reset = reset;
            try
            {
                AudioInputStream stream = AudioSystem.getAudioInputStream(WAV, 
                        AudioSystem.getAudioInputStream(new File("sounds/" + path + ".wav")));
                this.clip = (Clip) AudioSystem.getLine(INFO);
                clip.open(stream);
                clip.addLineListener((e) -> 
                {
                    if(e.getFramePosition() > 0)
                    {
                        clip.stop();
                        clip.setMicrosecondPosition(0);
                    }
                });
            }
            catch(Exception ex)
            {
                System.err.println("error with 'sounds/" + path + ".wav'");
                System.err.println(ex);
                this.clip = null;
            }
        }
        
        private void play()
        {
            if(clip != null)
            {
                if(reset)
                {
                    clip.stop();
                    clip.setMicrosecondPosition(0);
                }
                clip.start();
            }
        }
        
        private void stop()
        {
            if(clip != null)
            {
                clip.stop();
            }
        }
    }
    
    public static void playSound(Sound sound)
    {
        if(Game.get().isSoundEnabled() && sound != null)
        {
            /*if(sound != Sound.MENU_MUSIC && sound != Sound.SONG_1 && sound != Sound.WALK)
            {
                System.out.println("PLAYING " + sound);
            }*/
            sound.play();
        }
    }
    
    public static void stopSound(Sound sound)
    {
        sound.stop();
    }
    
    public static void turnSoundOff()
    {
        for(Sound sound : Sound.values())
        {
            sound.stop();
        }
    }
}
